import mock
import json
import query_client
import config
import utils
from sets import Set

API_LABELS = ['advisory_id', 'sir', 'first_published', 'last_updated', 'cves', 'cvss_base_score',
              'advisory_title', 'publication_url', 'cwe', 'product_names', 'summary',
              'oval_url', 'cvrf_url', 'bug_ids', 'ios_release', 'first_fixed', 'ips_signatures']
API_URL = "https://api.cisco.com/security/advisories"
ips_signature_no = """ "NA" """
ips_signature_yes = """{
        "legacyIpsId" : "test-val-ll",
        "releaseVersion" : "test-val-ll",
        "softwareVersion" : "test-val-ll",
        "legacyIpsUrl" : "test-val-ll"
        }"""
ips_signature_two = """{
        "legacyIpsId" : "test-val-ll",
        "releaseVersion" : "test-val-ll",
        "softwareVersion" : "test-val-ll",
        "legacyIpsUrl" : "test-val-ll"
},
{
        "legacyIpsId" : "test-val-32",
        "releaseVersion" : "test-val-l32",
        "softwareVersion" : "test-val-32",
        "legacyIpsUrl" : "test-val-32"
}"""

mock_response = """{
    "advisories": [
    {
      "advisoryId": "Cisco-SA-20111107-CVE-2011-0941",
      "sir": "Medium",
      "firstPublished": "2011-11-07T21:36:55+0000",
      "lastUpdated": "2011-11-07T21:36:55+0000",
      "cves": [
        "CVE-2011-0941"
      ],
      "cvrfUrl": "http://tools.cisco.com/security/center/contentxml/CiscoSecurityAdvisory/Cisco-SA-20111107-CVE-2011-0941/cvrf/Cisco-SA-20111107-CVE-2011-0941_cvrf.xml",
    "bugIDs": [
        "CSCtj09179"
      ],
      "cvssBaseScore": "7.8",
      "advisoryTitle": "Cisco IOS Software and Cisco Unified Communications Manager Session Initiation Protocol Packet Processing Memory Leak Vulnerability",
      "publicationUrl": "http://tools.cisco.com/security/center/content/CiscoSecurityAdvisory/Cisco-SA-20111107-CVE-2011-0941",
      "cwe": [
        "CWE-399",
        "CWE-101"
      ],
      "productNames": [
        "Cisco Unified Communications Manager (CallManager)",
        "Cisco IOS Software Releases 12.4 T",
        "Cisco IOS Software Release 12.4(2)T",
        "Cisco IOS Software Release 12.4(4)T",
        "Cisco IOS Software Release 12.4(6)T",
        "Cisco IOS Software Release 12.4(9)T",
        "Cisco IOS Software Release 12.4(11)T",
        "Cisco IOS Software Release 12.4(15)T",
        "Cisco IOS Software Release 12.4(20)T",
        "Cisco IOS Software Release 12.4(22)T",
        "Cisco Unified Communications Manager Version 7.1",
        "Cisco IOS Software Release 12.4(24)T",
        "Cisco IOS 15.1M&T",
        "Cisco IOS Software Release 15.1(1)T",
        "Cisco Unified Communications Manager Version 8.0",
        "Cisco IOS Software Release 15.1(2)T",
        "Cisco Unified Communications Manager Version 8.5",
        "Cisco IOS 15.1S",
        "Cisco IOS Software Release 15.1(3)T",
        "Cisco IOS Software Release 15.1(4)M",
        "Cisco IOS Software Release 15.1(1)S",
        "Cisco IOS Software Release 15.1(2)S",
        "Cisco IOS Software Release 15.1(3)S"
      ],
      "summary":"It's very bad",
      "ipsSignatures": [
        %s

       ]
    }
    ]
    }
    """ % ips_signature_yes


mock_response_multiple_adv = """{
    "advisories": [
    {
      "advisoryId": "Cisco-SA-20111107-CVE-2011-0941",
      "sir": "Medium",
      "firstPublished": "2011-11-07T21:36:55+0000",
      "lastUpdated": "2011-11-07T21:36:55+0000",
      "cves": [
        "CVE-2011-0941"
      ],
      "cvrfUrl": "http://tools.cisco.com/security/center/contentxml/CiscoSecurityAdvisory/Cisco-SA-20111107-CVE-2011-0941/cvrf/Cisco-SA-20111107-CVE-2011-0941_cvrf.xml",
    "bugIDs": [
        "CSCtj09179"
      ],
      "cvssBaseScore": "7.8",
      "advisoryTitle": "Cisco IOS Software and Cisco Unified Communications Manager Session Initiation Protocol Packet Processing Memory Leak Vulnerability",
      "publicationUrl": "http://tools.cisco.com/security/center/content/CiscoSecurityAdvisory/Cisco-SA-20111107-CVE-2011-0941",
      "cwe": [
        "CWE-399",
        "CWE-101"
      ],
      "productNames": [
        "Cisco Unified Communications Manager (CallManager)",
        "Cisco IOS Software Releases 12.4 T"
      ],
      "summary":"It's very bad",
      "ipsSignatures": [
        %s

       ]
    },
     {
      "advisoryId": "Cisco-SA-20111107-CVE-2011-0941",
      "sir": "Medium",
      "firstPublished": "2011-11-07T21:36:55+0000",
      "lastUpdated": "2011-11-07T21:36:55+0000",
      "cves": [
        "CVE-2011-0941"
      ],
      "cvrfUrl": "http://tools.cisco.com/security/center/contentxml/CiscoSecurityAdvisory/Cisco-SA-20111107-CVE-2011-0941/cvrf/Cisco-SA-20111107-CVE-2011-0941_cvrf.xml",
    "bugIDs": [
        "CSCtj09179"
      ],
      "cvssBaseScore": "7.8",
      "advisoryTitle": "Cisco IOS Software and Cisco Unified Communications Manager Session Initiation Protocol Packet Processing Memory Leak Vulnerability",
      "publicationUrl": "http://tools.cisco.com/security/center/content/CiscoSecurityAdvisory/Cisco-SA-20111107-CVE-2011-0941",
      "cwe": [
        "CWE-399",
        "CWE-101"
      ],
      "productNames": [
        "Cisco Unified Communications Manager (CallManager)",
        "Cisco IOS Software Releases 12.4 T"
      ],
      "summary":"It's very bad",
      "ipsSignatures": [
        %s

       ]
    }
    ]
    }""" %(ips_signature_two, ips_signature_no)


def mocked_get_requests(*args, **kwargs):
    """Mocks requests get method from QueryClient"""

    url = "{base_url}/{path}".format(base_url = API_URL,
                                     path = args[0])
    return mocked_requests_lib_get(url = url).json()

def mocked_requests_lib_get(*args, **kwargs):
    """Mocks library requests.get method"""
    class MockResponse():
        def __init__(self, status_code, json_response):
            self.status_code = status_code
            self.json_response = json_response

        def json(self):
            return json.loads(self.json_response)

        def raise_for_status(self):
            if self.status_code == 404:
                raise Exception("Mock 404 Not Found Exception")
            elif self.status_code == 406:
                raise Exception("Mock 406 HttpError Exception")

    return MockResponse(200, mock_response_multiple_adv)


@mock.patch("query_client.OpenVulnQueryClient.get_request", side_effect=mocked_get_requests)
def main(mocked_get_requests):
    qc = query_client.OpenVulnQueryClient(client_id=config.CLIENT_ID, client_secret=config.CLIENT_SECRET)
    advisories = qc.get_by_severity(adv_format="cvrf", severity="high")
    cvrf = advisories[0].ips_signatures

    # for each in utils.filter_advisories(advisories, API_LABELS):
    #     print each.keys()
    #     for key in  each.keys():
    #         print "%s => %s \n" % (key, each[key])

    utils.output(utils.filter_advisories(advisories, ['ips_signatures', 'advisory_id']), 'csv', utils.get_output_filehandle())


if __name__ == "__main__":
    main()
